from enum import Enum, auto, unique
from logging import getLogger
from typing import Hashable, Optional, Union

from chatushka.core.matchers.base import MatcherBase
from chatushka.core.models import MatchedToken
from chatushka.core.transports.models import Update

logger = getLogger(__name__)


@unique
class ChatUsersMovementsEventsEnum(Enum):
    CAME = auto()


class ChatUsersMovementsMatcher(MatcherBase):
    def _cast_token(
        self,
        token: Union[str, ChatUsersMovementsEventsEnum],
    ) -> ChatUsersMovementsEventsEnum:
        if isinstance(token, str):
            return ChatUsersMovementsEventsEnum[token.upper()]
        return token

    async def _check(
        self,
        token: Hashable,
        update: Update,
    ) -> Optional[MatchedToken]:
        if update.my_chat_member is None and update.message and update.message.text is None:
            return MatchedToken(token=ChatUsersMovementsEventsEnum.CAME)



from chatushka.bot.settings import get_settings
from chatushka.core.matchers import ChatUsersMovementsEventsEnum, ChatUsersMovementsMatcher
from chatushka.core.transports.models import Message
from chatushka.core.transports.telegram_bot_api import TelegramBotApi

settings = get_settings()
welcoming_matcher = ChatUsersMovementsMatcher()


@welcoming_matcher(ChatUsersMovementsEventsEnum.CAME, include_in_help=False)
async def came_handler(
    api: TelegramBotApi,
    message: Message,
) -> None:
    text = f'Hi, <a href="tg://user?id={message.user.id}">{message.user.readable_name}</a>!'
    await api.send_message(
        chat_id=message.chat.id,
        text=text,
    )
